@@ -1,5 +1,4 @@
 
-
 #include <ATen/cuda/CUDAContext.h>
 #include <torch/extension.h>
 #include <vector>
@@ -15,13 +14,22 @@ at::Tensor ds_softmax(at::Tensor& attn_scores,
                       bool triangular,
                       bool recompute,
                       bool local_attention,
-                      int window_size)
+                      int window_size,
+                      bool async_op)
 {
     auto attn_scores_c = attn_scores.contiguous();
     int bsz = attn_scores_c.size(0);
-    int seq_len = attn_scores_c.size(2);
-    int soft_len = attn_scores_c.size(3);
-    int heads = attn_scores_c.size(1);
+
+    int seq_len = attn_scores_c.size(1);
+    int len = attn_scores_c.sizes().size();
+    if (len > 3) seq_len = attn_scores_c.size(2);
+
+    int soft_len = attn_scores_c.size(2);
+    if (len > 3) soft_len = attn_scores_c.size(3);
+
+    int heads = 1;
+    if (len > 3) heads = attn_scores_c.size(1);
+
     launch_attn_softmax_v2((T*)attn_scores_c.data_ptr(),
                            (attn_mask.sizes().size() > 1 ? (T*)attn_mask.data_ptr() : nullptr),
                            triangular,
@@ -33,11 +41,63 @@ at::Tensor ds_softmax(at::Tensor& attn_scores,
                            seq_len,
                            soft_len,
                            1.0,
-                           at::cuda::getCurrentCUDAStream());
+                           Context::Instance().GetCurrentStream(async_op));
 
     return attn_scores_c;
 }
 
+template <typename T>
+void allocate_workspace(size_t hidden_dim,
+                        size_t max_seq_len,
+                        size_t batch_size,
+                        unsigned num_layers,
+                        size_t head_size = 128)
+{
+    size_t _workSpaceSize = 16 * (hidden_dim * batch_size * max_seq_len) +
+                            (num_layers * batch_size * max_seq_len * hidden_dim * 2);  // KV-cache
+    Context::Instance().GenWorkSpace(_workSpaceSize * sizeof(T));
+}
+
+template <typename T>
+at::Tensor einsum_sec_sm_ecm(at::Tensor& Q, at::Tensor& W)
+{
+    auto options = at::TensorOptions()
+                       .dtype(Q.options().dtype())
+                       .layout(at::kStrided)
+                       .device(at::kCUDA)
+                       .requires_grad(false);
+    T* workspace = (T*)Context::Instance().GetWorkSpace();
+    float alpha = 1;
+    float gemm_beta = 0.0;
+
+    if (!workspace) {
+        allocate_workspace<T>(W.size(1), MAX_OUT_TOKES, Q.size(0), 1);
+        workspace = (T*)Context::Instance().GetWorkSpace();
+    }
+
+    auto O = at::from_blob(workspace, {Q.size(1), Q.size(2), W.size(1)}, options);
+    unsigned m = W.size(1);
+    unsigned n = Q.size(1) * Q.size(2);
+    unsigned k = Q.size(0);
+    cublas_gemm_ex(Context::Instance().GetCublasHandle(),
+                   CUBLAS_OP_N,
+                   CUBLAS_OP_T,
+                   m,
+                   n,
+                   k,
+                   &alpha,
+                   &gemm_beta,
+                   (T*)W.data_ptr(),
+                   (T*)Q.data_ptr(),
+                   (T*)O.data_ptr(),
+#ifdef __HIP_PLATFORM_HCC__
+                   rocblas_gemm_algo_standard);
+#else
+                   CUBLAS_GEMM_DEFAULT_TENSOR_OP);
+#endif
+    return O;
+}
+
 template <typename T>
 void attention_unfused(at::Tensor& prev_key_cont,
                        at::Tensor& query_cont,
@@ -61,7 +121,7 @@ void attention_unfused(at::Tensor& prev_key_cont,
                        .requires_grad(false);
     float alpha = norm_factor;
     float gemm_beta = 0.0;
-    auto attn_score = at::zeros({bsz, heads, seq_len, soft_len}, options);
+    auto attn_score = at::empty({bsz, heads, seq_len, soft_len}, options);
     int k = prev_value_cont.size(2) / heads;
     cublasSetStream(Context::Instance().GetCublasHandle(), Context::Instance().GetCurrentStream());
     cublas_strided_batched_gemm(Context::Instance().GetCublasHandle(),
@@ -79,9 +139,13 @@ void attention_unfused(at::Tensor& prev_key_cont,
                                 seq_len * k,
                                 seq_len * soft_len,
                                 bsz * heads,
+#ifdef __HIP_PLATFORM_HCC__
+                                rocblas_gemm_algo_standard);
+#else
                                 CUBLAS_GEMM_DEFAULT_TENSOR_OP);
-    attn_score =
-        ds_softmax<T>(attn_score, attn_mask, triangular, recompute, local_attention, window_size);
+#endif
+    attn_score = ds_softmax<T>(
+        attn_score, attn_mask, triangular, recompute, local_attention, window_size, false);
     alpha = 1.0;
     cublas_strided_batched_gemm(Context::Instance().GetCublasHandle(),
                                 k,
@@ -98,23 +162,27 @@ void attention_unfused(at::Tensor& prev_key_cont,
                                 seq_len * soft_len,
                                 seq_len * k,
                                 bsz * heads,
+#ifdef __HIP_PLATFORM_HCC__
+                                rocblas_gemm_algo_standard);
+#else
                                 CUBLAS_GEMM_DEFAULT_TENSOR_OP);
+#endif
 }
 
 template <typename T>
-std::vector<at::Tensor> ds_softmax_context(at::Tensor& query,
-                                           at::Tensor& prev_key,
-                                           at::Tensor& new_key,
-                                           at::Tensor& attn_mask,
-                                           at::Tensor& prev_value,
-                                           at::Tensor& new_value,
-                                           int heads,
-                                           float norm_factor,
-                                           bool merging,
-                                           bool triangular,
-                                           bool local_attention,
-                                           int window_size,
-                                           bool no_masking)
+std::vector<at::Tensor> ds_softmax_context1(at::Tensor& query,
+                                            at::Tensor& prev_key,
+                                            at::Tensor& new_key,
+                                            at::Tensor& attn_mask,
+                                            at::Tensor& prev_value,
+                                            at::Tensor& new_value,
+                                            int heads,
+                                            float norm_factor,
+                                            bool merging,
+                                            bool triangular,
+                                            bool local_attention,
+                                            int window_size,
+                                            bool no_masking)
 {
     auto query_cont = query.contiguous();
     auto prev_key_cont = prev_key.contiguous();
@@ -154,6 +222,211 @@ std::vector<at::Tensor> ds_softmax_context(at::Tensor& query,
     return {output, prev_key, prev_value};
 }
 
+template <typename T>
+void ds_softmax_internal(T* attn_scores,
+                         at::Tensor& attn_mask,
+                         bool triangular,
+                         bool recompute,
+                         bool local_attention,
+                         int window_size,
+                         int bsz,
+                         int seq_len,
+                         int soft_len,
+                         int heads)
+{
+    launch_attn_softmax_v2((T*)attn_scores,
+                           (attn_mask.sizes().size() > 1 ? (T*)attn_mask.data_ptr() : nullptr),
+                           triangular,
+                           recompute,
+                           local_attention,
+                           window_size,
+                           bsz,
+                           heads,
+                           seq_len,
+                           soft_len,
+                           1.0,
+                           at::cuda::getCurrentCUDAStream());
+}
+
+template <typename T>
+void attention_unfused(T* prev_key_cont,
+                       T* query_cont,
+                       at::Tensor& attn_mask,
+                       T* prev_value_cont,
+                       T* output,
+                       unsigned& bsz,
+                       int& k,
+                       unsigned& seq_len,
+                       unsigned& soft_len,
+                       int& heads,
+                       float& norm_factor,
+                       bool triangular,
+                       bool recompute,
+                       bool local_attention,
+                       int window_size)
+{
+    float alpha = norm_factor * norm_factor;
+    float gemm_beta = 0.0;
+    T* workspace = (T*)output + bsz * seq_len * heads * k;
+
+    cublasSetStream(Context::Instance().GetCublasHandle(), Context::Instance().GetCurrentStream());
+    cublas_strided_batched_gemm(Context::Instance().GetCublasHandle(),
+                                soft_len,
+                                seq_len,
+                                k,
+                                &alpha,
+                                &gemm_beta,
+                                (T*)prev_key_cont,
+                                (T*)query_cont,
+                                workspace,
+                                CUBLAS_OP_T,
+                                CUBLAS_OP_N,
+                                MAX_OUT_TOKES * k,
+                                seq_len * k,
+                                seq_len * soft_len,
+                                bsz * heads,
+#ifdef __HIP_PLATFORM_HCC__
+                                rocblas_gemm_algo_standard);
+#else
+                                CUBLAS_GEMM_DEFAULT_TENSOR_OP);
+#endif
+    ds_softmax_internal<T>(workspace,
+                           attn_mask,
+                           triangular,
+                           recompute,
+                           local_attention,
+                           window_size,
+                           bsz,
+                           seq_len,
+                           soft_len,
+                           heads);
+    alpha = 1.0;
+    cublas_strided_batched_gemm(Context::Instance().GetCublasHandle(),
+                                k,
+                                seq_len,
+                                soft_len,
+                                &alpha,
+                                &gemm_beta,
+                                (T*)prev_value_cont,
+                                workspace,
+                                (T*)output,
+                                CUBLAS_OP_N,
+                                CUBLAS_OP_N,
+                                MAX_OUT_TOKES * k,
+                                seq_len * soft_len,
+                                seq_len * k,
+                                bsz * heads,
+#ifdef __HIP_PLATFORM_HCC__
+                                rocblas_gemm_algo_standard);
+#else
+                                CUBLAS_GEMM_DEFAULT_TENSOR_OP);
+#endif
+}
+
+template <typename T>
+std::vector<at::Tensor> ds_softmax_context(at::Tensor& query_key_value,
+                                           at::Tensor& attn_mask,
+                                           int rotary_dim,
+                                           bool rotate_half,
+                                           bool rotate_every_two,
+                                           int heads,
+                                           float norm_factor,
+                                           bool triangular,
+                                           bool local_attention,
+                                           int window_size,
+                                           bool no_masking,
+                                           unsigned layer_id,
+                                           unsigned num_layers)
+{
+    unsigned bsz = query_key_value.size(0);
+    unsigned seq_len = query_key_value.size(1);
+    unsigned hidden_dim = query_key_value.size(2) / 3;
+
+    bool is_prompt = (seq_len > 1);
+
+    if (is_prompt) Context::Instance().reset_tokens(seq_len);
+    unsigned soft_len = Context::Instance().current_tokens();
+
+    int k = hidden_dim / heads;
+    auto options = at::TensorOptions()
+                       .dtype(query_key_value.options().dtype())
+                       .layout(at::kStrided)
+                       .device(at::kCUDA)
+                       .requires_grad(false);
+
+    T* workspace = (T*)Context::Instance().GetWorkSpace();
+    size_t buf_size = bsz * seq_len * hidden_dim;
+    auto output = torch::from_blob(workspace + 4 * buf_size, {bsz, seq_len, hidden_dim}, options);
+
+    auto query_cont = workspace + 8 * buf_size;
+    size_t offset =
+        16 * (hidden_dim * bsz * MAX_OUT_TOKES) + layer_id * 2 * bsz * MAX_OUT_TOKES * hidden_dim;
+
+    unsigned all_tokens = soft_len;
+    auto kv_cache = workspace + offset + (hidden_dim / heads) * (is_prompt ? 0 : soft_len - 1);
+    size_t value_offset = bsz * MAX_OUT_TOKES * hidden_dim;
+
+    T* temp_buf = (T*)output.data_ptr() + at::numel(output);
+    launch_bias_add_transform_0213<T>((T*)query_cont,
+                                      kv_cache,
+                                      kv_cache + value_offset,
+                                      (T*)query_key_value.data_ptr(),
+                                      nullptr,
+                                      bsz,
+                                      seq_len,
+                                      (is_prompt ? 0 : soft_len - 1),
+                                      soft_len,
+                                      hidden_dim,
+                                      heads,
+                                      rotary_dim,
+                                      rotate_half,
+                                      rotate_every_two,
+                                      Context::Instance().GetCurrentStream(),
+                                      3);
+    if (rotary_dim > 0 && rotate_half)
+        launch_apply_rotary_pos_emb(query_cont,
+                                    kv_cache,
+                                    k,
+                                    seq_len,
+                                    rotary_dim,
+                                    (is_prompt ? 0 : soft_len - 1),
+                                    heads,
+                                    bsz,
+                                    rotate_half,
+                                    rotate_every_two,
+                                    Context::Instance().GetCurrentStream());
+
+    attention_unfused<T>(workspace + offset,
+                         (T*)query_cont,
+                         attn_mask,
+                         workspace + offset + value_offset,
+                         temp_buf,
+                         bsz,
+                         k,
+                         seq_len,
+                         all_tokens,
+                         heads,
+                         norm_factor,
+                         (triangular && is_prompt),
+                         is_prompt,
+                         local_attention,
+                         window_size);
+    launch_transform4d_0213<T>((T*)output.data_ptr(),
+                               temp_buf,
+                               bsz,
+                               heads,
+                               seq_len,
+                               output.size(2),
+                               Context::Instance().GetCurrentStream(false),
+                               1);
+
+    if (layer_id == num_layers - 1) Context::Instance().advance_tokens();
+    auto prev_key = torch::from_blob(workspace + offset, {bsz, all_tokens, hidden_dim}, options);
+    auto prev_value =
+        torch::from_blob(workspace + offset + value_offset, {bsz, all_tokens, hidden_dim}, options);
+    return {output, prev_key, prev_value};
+}
+
 template <typename T>
 at::Tensor ds_bias_gelu(at::Tensor& input, at::Tensor& bias)
 {
@@ -177,13 +450,13 @@ at::Tensor ds_bias_residual(at::Tensor& input, at::Tensor& residual, at::Tensor&
     auto residual_cont = residual.contiguous();
 
     int bsz = input_cont.size(0) * input_cont.size(1);
-
-    launch_bias_residual((T*)input_cont.data_ptr(),
-                         (T*)residual_cont.data_ptr(),
-                         (T*)bias.data_ptr(),
-                         bsz,
-                         input_cont.size(2),
-                         Context::Instance().GetCurrentStream());
+    // launch_bias_residual((T*)input_cont.data_ptr(),
+    //                      (T*)residual_cont.data_ptr(),
+    //                      (T*)bias.data_ptr(),
+    //                      bsz,
+    //                      input_cont.size(2),
+    //                      (bias.size(0) > 1),
+    //                      Context::Instance().GetCurrentStream());
     return input_cont;
 }
 
@@ -203,6 +476,24 @@ at::Tensor ds_layernorm(at::Tensor& input_cont, at::Tensor& gamma, at::Tensor& b
     return inp_norm;
 }
 
+template <typename T>
+void ds_layernorm_internal(T* workspace,
+                           at::Tensor& input,
+                           at::Tensor& gamma,
+                           at::Tensor& betta,
+                           float epsilon)
+{
+    int bsz = input.size(0) * input.size(1);
+    launch_layer_norm(workspace,
+                      (T*)input.data_ptr(),
+                      (T*)gamma.data_ptr(),
+                      (T*)betta.data_ptr(),
+                      epsilon,
+                      bsz,
+                      input.size(2),
+                      Context::Instance().GetCurrentStream());
+}
+
 template <typename T>
 at::Tensor qkv_unfused_cublas(at::Tensor& output,
                               at::Tensor& input,
@@ -213,13 +504,15 @@ at::Tensor qkv_unfused_cublas(at::Tensor& output,
                               const float epsilon,
                               bool add_bias)
 {
-    auto inp_norm = ds_layernorm<T>(input, gamma, beta, epsilon);
-
+    int bsz = input.size(0) * input.size(1);
+    T* workspace = (T*)Context::Instance().GetWorkSpace();
+    workspace += (3 * input.size(0) * MAX_OUT_TOKES * input.size(2));
+    ds_layernorm_internal<T>(workspace, input, gamma, beta, epsilon);
     // cudaEventRecord(Context::Instance().GetCompEvent(1), Context::Instance().GetCurrentStream());
 
     float alpha = (T)1.0;
     float gemm_beta = (T)0.0;
-    int bsz = input.size(0) * input.size(1);
+
     cublasSetStream(Context::Instance().GetCublasHandle(), Context::Instance().GetCurrentStream());
     cublas_gemm_ex(Context::Instance().GetCublasHandle(),
                    CUBLAS_OP_N,
@@ -230,16 +523,21 @@ at::Tensor qkv_unfused_cublas(at::Tensor& output,
                    &alpha,
                    &gemm_beta,
                    (T*)weight.data_ptr(),
-                   (T*)inp_norm.data_ptr(),
+                   workspace,
                    (T*)output.data_ptr(),
+#ifdef __HIP_PLATFORM_HCC__
+                   rocblas_gemm_algo_standard);
+#else
                    CUBLAS_GEMM_DEFAULT_TENSOR_OP);
+#endif
     if (add_bias)
         launch_bias_add((T*)output.data_ptr(),
                         (T*)bias.data_ptr(),
                         weight.size(1),
                         bsz,
                         Context::Instance().GetCurrentStream());
-    return inp_norm;
+
+    return torch::from_blob(workspace, input.sizes(), input.options());
 }
 
 template <typename T>
@@ -249,19 +547,27 @@ std::vector<at::Tensor> ds_qkv_gemm(at::Tensor& input,
                                     at::Tensor& gamma,
                                     at::Tensor& beta,
                                     const float epsilon,
-                                    bool add_bias)
+                                    bool add_bias,
+                                    unsigned num_layers)
 {
-    auto input_cont = input.contiguous();
+    int bsz = input.size(0) * input.size(1);
+    int out_size = weight.size(1);
+    T* workspace = (T*)Context::Instance().GetWorkSpace();
+    if (!workspace) {
+        cublasSetStream(Context::Instance().GetCublasHandle(),
+                        Context::Instance().GetCurrentStream());
+        allocate_workspace<T>(input.size(2), MAX_OUT_TOKES, input.size(0), num_layers);
+        workspace = (T*)Context::Instance().GetWorkSpace();
+    }
     auto options = at::TensorOptions()
-                       .dtype(input_cont.options().dtype())
+                       .dtype(input.options().dtype())
                        .layout(at::kStrided)
                        .device(at::kCUDA)
                        .requires_grad(false);
 
-    auto output = at::empty({input_cont.size(0), input_cont.size(1), weight.size(1)}, options);
-    int bsz = input_cont.size(0) * input_cont.size(1);
+    auto output = at::from_blob(workspace, {input.size(0), input.size(1), weight.size(1)}, options);
     auto inp_norm =
-        qkv_unfused_cublas<T>(output, input_cont, weight, bias, gamma, beta, epsilon, add_bias);
+        qkv_unfused_cublas<T>(output, input, weight, bias, gamma, beta, epsilon, add_bias);
 
     return {output, inp_norm};
 }
@@ -306,7 +612,11 @@ void quantized_gemm(at::Tensor& output,
                    (T*)weight16.data_ptr(),
                    (T*)input.data_ptr(),
                    (T*)output.data_ptr(),
+#ifdef __HIP_PLATFORM_HCC__
+                   rocblas_gemm_algo_standard);
+#else
                    CUBLAS_GEMM_DEFAULT_TENSOR_OP);
+#endif
 }
 
 template <typename T>
@@ -371,7 +681,11 @@ at::Tensor ds_linear_layer(at::Tensor& input, at::Tensor& weight, at::Tensor& bi
                    (T*)weight.data_ptr(),
                    (T*)input_cont.data_ptr(),
                    (T*)output.data_ptr(),
+#ifdef __HIP_PLATFORM_HCC__
+                   rocblas_gemm_algo_standard);
+#else
                    CUBLAS_GEMM_DEFAULT_TENSOR_OP);
+#endif
 
     launch_bias_add((T*)output.data_ptr(),
                     (T*)bias.data_ptr(),
@@ -409,7 +723,7 @@ at::Tensor ds_linear_layer_int8(at::Tensor& input,
 }
 
 template <typename T>
-at::Tensor ds_vector_matmul(at::Tensor& input, at::Tensor& weight)
+at::Tensor ds_vector_matmul(at::Tensor& input, at::Tensor& weight, bool async_op)
 {
     auto input_cont = input.contiguous();
     auto options = at::TensorOptions()
@@ -422,7 +736,8 @@ at::Tensor ds_vector_matmul(at::Tensor& input, at::Tensor& weight)
     int bsz = input_cont.size(0) * input_cont.size(1);
     float alpha = (T)1.0;
     float gemm_beta = (T)0.0;
-    cublasSetStream(Context::Instance().GetCublasHandle(), Context::Instance().GetCurrentStream());
+    cublasSetStream(Context::Instance().GetCublasHandle(),
+                    Context::Instance().GetCurrentStream(async_op));
     cublas_gemm_ex(Context::Instance().GetCublasHandle(),
                    CUBLAS_OP_N,
                    CUBLAS_OP_N,
@@ -434,8 +749,11 @@ at::Tensor ds_vector_matmul(at::Tensor& input, at::Tensor& weight)
                    (T*)weight.data_ptr(),
                    (T*)input_cont.data_ptr(),
                    (T*)output.data_ptr(),
+#ifdef __HIP_PLATFORM_HCC__
+                   rocblas_gemm_algo_standard);
+#else
                    CUBLAS_GEMM_DEFAULT_TENSOR_OP);
-
+#endif
     return output;
 }
 
@@ -461,7 +779,6 @@ at::Tensor ds_vector_matmul_int8(at::Tensor& input,
 
 template <typename T>
 void mlp_unfused_cublas(at::Tensor& output,
-                        at::Tensor& residual_add,
                         at::Tensor& input,
                         at::Tensor& residual,
                         at::Tensor& input_bias,
@@ -470,13 +787,14 @@ void mlp_unfused_cublas(at::Tensor& output,
                         at::Tensor& gamma,
                         at::Tensor& beta,
                         const float epsilon,
-                        bool preLayerNorm)
+                        bool preLayerNorm,
+                        bool mlp_after_attn)
 {
     int bsz = input.size(0) * input.size(1);
-    auto inp_norm = preLayerNorm ? at::empty_like(input) : residual_add;
+    auto inp_norm = at::empty_like(input);
 
     launch_residual_layer_norm((T*)inp_norm.data_ptr(),
-                               (T*)residual_add.data_ptr(),
+                               (T*)nullptr,
                                (T*)input.data_ptr(),
                                (T*)residual.data_ptr(),
                                (T*)input_bias.data_ptr(),
@@ -486,6 +804,7 @@ void mlp_unfused_cublas(at::Tensor& output,
                                bsz,
                                input.size(2),
                                preLayerNorm,
+                               mlp_after_attn,
                                Context::Instance().GetCurrentStream());
 
     float alpha = (T)1.0;
@@ -502,8 +821,11 @@ void mlp_unfused_cublas(at::Tensor& output,
                    (T*)weight.data_ptr(),
                    (T*)inp_norm.data_ptr(),
                    (T*)output.data_ptr(),
+#ifdef __HIP_PLATFORM_HCC__
+                   rocblas_gemm_algo_standard);
+#else
                    CUBLAS_GEMM_DEFAULT_TENSOR_OP);
-
+#endif
     launch_bias_gelu((T*)output.data_ptr(),
                      (T*)bias.data_ptr(),
                      weight.size(1),
@@ -511,15 +833,16 @@ void mlp_unfused_cublas(at::Tensor& output,
                      Context::Instance().GetCurrentStream());
 }
 template <typename T>
-std::vector<at::Tensor> ds_mlp_gemm(at::Tensor& input,
-                                    at::Tensor& residual,
-                                    at::Tensor& input_bias,
-                                    at::Tensor& weight,
-                                    at::Tensor& bias,
-                                    at::Tensor& gamma,
-                                    at::Tensor& beta,
-                                    const float epsilon,
-                                    bool preLayerNorm)
+at::Tensor ds_mlp_gemm(at::Tensor& input,
+                       at::Tensor& residual,
+                       at::Tensor& input_bias,
+                       at::Tensor& weight,
+                       at::Tensor& bias,
+                       at::Tensor& gamma,
+                       at::Tensor& beta,
+                       const float epsilon,
+                       bool preLayerNorm,
+                       bool mlp_after_attn)
 {
     auto input_cont = input.contiguous();
     auto options = at::TensorOptions()
@@ -529,12 +852,10 @@ std::vector<at::Tensor> ds_mlp_gemm(at::Tensor& input,
                        .requires_grad(false);
 
     auto output = at::empty({input_cont.size(0), input_cont.size(1), weight.size(1)}, options);
-    auto residual_add = at::empty_like(input_cont);
     int bsz = input_cont.size(0) * input_cont.size(1);
 
     mlp_unfused_cublas<T>(output,
-                          residual_add,
-                          input,
+                          mlp_after_attn ? input : residual,
                           residual,
                           input_bias,
                           weight,
@@ -542,9 +863,10 @@ std::vector<at::Tensor> ds_mlp_gemm(at::Tensor& input,
                           gamma,
                           beta,
                           epsilon,
-                          preLayerNorm);
+                          preLayerNorm,
+                          mlp_after_attn);
 
-    return {output, residual_add};
+    return output;
 }
 
 template <typename T>
@@ -574,18 +896,18 @@ std::vector<at::Tensor> ds_mlp_gemm_int8(at::Tensor& input,
 
     auto residual_add = (preLayerNorm ? at::empty_like(input_cont) : inp_norm);
     // computing the blocking across K dimension
-    launch_residual_layer_norm((T*)inp_norm.data_ptr(),
-                               (T*)residual_add.data_ptr(),
-                               (T*)input_cont.data_ptr(),
-                               (T*)residual.data_ptr(),
-                               (T*)input_bias.data_ptr(),
-                               (T*)gamma.data_ptr(),
-                               (T*)beta.data_ptr(),
-                               epsilon,
-                               bsz,
-                               input_cont.size(2),
-                               preLayerNorm,
-                               Context::Instance().GetCurrentStream());
+    // launch_residual_layer_norm((T*)inp_norm.data_ptr(),
+    //                           (T*)residual_add.data_ptr(),
+    //                           (T*)input_cont.data_ptr(),
+    //                           (T*)residual.data_ptr(),
+    //                           (T*)input_bias.data_ptr(),
+    //                           (T*)gamma.data_ptr(),
+    //                           (T*)beta.data_ptr(),
+    //                           epsilon,
+    //                           bsz,
+    //                           input_cont.size(2),
+    //                           preLayerNorm,
+    //                           Context::Instance().GetCurrentStream());
 
     quantized_gemm<T>(output, inp_norm, weight, q_scale, groups, 0);
     launch_bias_gelu((T*)output.data_ptr(),
@@ -601,10 +923,11 @@ template <typename T>
 at::Tensor fused_gemm_gelu(at::Tensor& input,
                            at::Tensor& weight,
                            at::Tensor& bias,
-                           at::Tensor& weight_out)
+                           at::Tensor& weight_out,
+                           const float epsilon,
+                           bool preLayerNorm,
+                           bool async_op)
 {
-    // cudaStreamWaitEvent(
-    //    Context::Instance().GetCurrentStream(true), Context::Instance().GetCompEvent(1), 0);
     auto input_cont = input.contiguous();
     auto options = at::TensorOptions()
                        .dtype(input_cont.options().dtype())
@@ -630,7 +953,11 @@ at::Tensor fused_gemm_gelu(at::Tensor& input,
                    (T*)weight.data_ptr(),
                    (T*)input_cont.data_ptr(),
                    (T*)intermediate.data_ptr(),
+#ifdef __HIP_PLATFORM_HCC__
+                   rocblas_gemm_algo_standard);
+#else
                    CUBLAS_GEMM_DEFAULT_TENSOR_OP);
+#endif
     launch_bias_gelu((T*)intermediate.data_ptr(),
                      (T*)bias.data_ptr(),
                      weight.size(1),
@@ -648,36 +975,69 @@ at::Tensor fused_gemm_gelu(at::Tensor& input,
                    (T*)weight_out.data_ptr(),
                    (T*)intermediate.data_ptr(),
                    (T*)output.data_ptr(),
+#ifdef __HIP_PLATFORM_HCC__
+                   rocblas_gemm_algo_standard);
+#else
                    CUBLAS_GEMM_DEFAULT_TENSOR_OP);
+#endif
     // cudaEventRecord(Context::Instance().GetCompEvent(2),
     //                Context::Instance().GetCurrentStream(true));
     return output;
 }
 
-void gptj_residual_add(at::Tensor& output,
+void residual_add_bias(at::Tensor& output,
                        at::Tensor& input,
                        at::Tensor& attention_output,
-                       at::Tensor& output_b)
+                       at::Tensor& output_b,
+                       at::Tensor& attention_b,
+                       int mp_size,
+                       bool mlp_after_attn,
+                       bool add_bias)
 {
     int bsz = input.size(0) * input.size(1);
     int hidden_size = input.size(2);
     // cudaStreamWaitEvent(
     //    Context::Instance().GetCurrentStream(), Context::Instance().GetCompEvent(2), 0);
     if (input.scalar_type() == at::kFloat)
-        launch_gptj_residual_add<float>((float*)input.data_ptr(),
-                                        (float*)output.data_ptr(),
-                                        (float*)attention_output.data_ptr(),
-                                        (float*)output_b.data_ptr(),
-                                        hidden_size,
-                                        bsz,
-                                        Context::Instance().GetCurrentStream());
+        if (mlp_after_attn)
+            launch_bias_residual((float*)input.data_ptr(),
+                                 (float*)output.data_ptr(),
+                                 (float*)attention_output.data_ptr(),
+                                 (float*)output_b.data_ptr(),
+                                 (float*)attention_b.data_ptr(),
+                                 bsz,
+                                 hidden_size,
+                                 mp_size,
+                                 Context::Instance().GetCurrentStream());
+        else
+            launch_gptj_residual_add<float>((float*)input.data_ptr(),
+                                            (float*)output.data_ptr(),
+                                            (float*)attention_output.data_ptr(),
+                                            (float*)output_b.data_ptr(),
+                                            (float*)(add_bias ? attention_b.data_ptr() : nullptr),
+                                            hidden_size,
+                                            bsz,
+                                            mp_size,
+                                            Context::Instance().GetCurrentStream());
+    else if (mlp_after_attn)
+        launch_bias_residual((__half*)input.data_ptr(),
+                             (__half*)output.data_ptr(),
+                             (__half*)attention_output.data_ptr(),
+                             (__half*)output_b.data_ptr(),
+                             (__half*)attention_b.data_ptr(),
+                             bsz,
+                             hidden_size,
+                             mp_size,
+                             Context::Instance().GetCurrentStream());
     else
         launch_gptj_residual_add<__half>((__half*)input.data_ptr(),
                                          (__half*)output.data_ptr(),
                                          (__half*)attention_output.data_ptr(),
                                          (__half*)output_b.data_ptr(),
+                                         (__half*)(add_bias ? attention_b.data_ptr() : nullptr),
                                          hidden_size,
                                          bsz,
+                                         mp_size,
                                          Context::Instance().GetCurrentStream());
 }
 
@@ -685,7 +1045,9 @@ std::vector<at::Tensor> apply_rotary_pos_emb(at::Tensor& mixed_query,
                                              at::Tensor& key_layer,
                                              unsigned rotary_dim,
                                              unsigned offset,
-                                             unsigned num_heads)
+                                             unsigned num_heads,
+                                             bool rotate_half,
+                                             bool rotate_every_two)
 {
     auto query_cont = mixed_query.contiguous();
     auto key_cont = key_layer.contiguous();
@@ -703,6 +1065,8 @@ std::vector<at::Tensor> apply_rotary_pos_emb(at::Tensor& mixed_query,
                                            offset,
                                            num_heads,
                                            bsz,
+                                           rotate_half,
+                                           rotate_every_two,
                                            Context::Instance().GetCurrentStream());
     else
         launch_apply_rotary_pos_emb<__half>((__half*)query_cont.data_ptr(),
@@ -713,10 +1077,65 @@ std::vector<at::Tensor> apply_rotary_pos_emb(at::Tensor& mixed_query,
                                             offset,
                                             num_heads,
                                             bsz,
+                                            rotate_half,
+                                            rotate_every_two,
                                             Context::Instance().GetCurrentStream());
     return {query_cont, key_cont};
 }
 
+template <typename T>
+at::Tensor fused_gemm_gelu_int8(at::Tensor& input,
+                                at::Tensor& weight,
+                                at::Tensor& bias,
+                                const float epsilon,
+                                at::Tensor& q_scale,
+                                int groups,
+                                bool preLayerNorm)
+{
+    auto input_cont = input.contiguous();
+    auto options = at::TensorOptions()
+                       .dtype(input_cont.options().dtype())
+                       .layout(at::kStrided)
+                       .device(at::kCUDA)
+                       .requires_grad(false);
+
+    auto output = at::empty({input_cont.size(0), input_cont.size(1), weight.size(1)}, options);
+
+    int bsz = input_cont.size(0) * input_cont.size(1);
+
+    quantized_gemm<T>(output, input_cont, weight, q_scale, groups, 0);
+    launch_bias_gelu((T*)output.data_ptr(),
+                     (T*)bias.data_ptr(),
+                     weight.size(1),
+                     bsz,
+                     Context::Instance().GetCurrentStream());
+
+    return output;
+}
+
+at::Tensor moe_res_matmul(at::Tensor& moe_res, at::Tensor& coef, at::Tensor& output)
+{
+    int M = moe_res.size(0) * moe_res.size(1);
+    int N = moe_res.size(2);
+    Context::Instance().SynchComm();
+    if (moe_res.scalar_type() == at::kFloat) {
+        launch_moe_res_matmul<float>((float*)moe_res.data_ptr(),
+                                     (float*)coef.data_ptr(),
+                                     (float*)output.data_ptr(),
+                                     M,
+                                     N,
+                                     at::cuda::getCurrentCUDAStream());
+    } else {
+        launch_moe_res_matmul<__half>((__half*)moe_res.data_ptr(),
+                                      (__half*)coef.data_ptr(),
+                                      (__half*)output.data_ptr(),
+                                      M,
+                                      N,
+                                      at::cuda::getCurrentCUDAStream());
+    }
+    return output;
+}
+
 PYBIND11_MODULE(TORCH_EXTENSION_NAME, m)
 {
     m.def("softmax_fp32", &ds_softmax<float>, "DeepSpeed SoftMax with fp32 (CUDA)");
@@ -726,6 +1145,9 @@ PYBIND11_MODULE(TORCH_EXTENSION_NAME, m)
     m.def("softmax_context_fp16",
           &ds_softmax_context<__half>,
           "DeepSpeed attention with fp32 (CUDA)");
+    m.def("softmax_context_int8",
+          &ds_softmax_context1<__half>,
+          "DeepSpeed attention with fp32 (CUDA)");
     m.def("bias_gelu_fp32", &ds_bias_gelu<float>, "DeepSpeed Gelu with fp32 (CUDA)");
     m.def("bias_gelu_fp16", &ds_bias_gelu<__half>, "DeepSpeed Gelu with fp32 (CUDA)");
     m.def("bias_residual_fp32",
@@ -754,6 +1176,14 @@ PYBIND11_MODULE(TORCH_EXTENSION_NAME, m)
           "DeepSpeed linear_layer with int8 (CUDA)");
     m.def("fused_gemm_gelu_fp32", &fused_gemm_gelu<float>, "DeepSpeed mlp with fp32 (CUDA)");
     m.def("fused_gemm_gelu_fp16", &fused_gemm_gelu<__half>, "DeepSpeed mlp with fp16 (CUDA)");
-    m.def("gptj_residual_add", &gptj_residual_add, "DeepSpeed mlp with fp16 (CUDA)");
+    m.def("residual_add", &residual_add_bias, "DeepSpeed mlp with fp16 (CUDA)");
     m.def("apply_rotary_pos_emb", &apply_rotary_pos_emb, "DeepSpeed mlp with fp16 (CUDA)");
+    m.def("einsum_sec_sm_ecm_fp32",
+          &einsum_sec_sm_ecm<float>,
+          "DeepSpeed vector-MM with fp32 (CUDA)");
+
+    m.def("einsum_sec_sm_ecm_fp16",
+          &einsum_sec_sm_ecm<__half>,
+          "DeepSpeed vector-MM with fp16 (CUDA)");
+    m.def("moe_res_matmul", &moe_res_matmul, "DeepSpeed moe residual matmul (CUDA)");
 }
