@@ -1,6 +1,15 @@
 #pragma once
 
+#ifdef __HIP_PLATFORM_HCC__
+#define HALF_PRECISION_AVAILABLE = 1
+#include <hip/hip_cooperative_groups.h>
+#else
+#if __CUDA_ARCH__ >= 700
+#define HALF_PRECISION_AVAILABLE = 1
+#endif
 #include <cooperative_groups.h>
+#endif
+
 #include <cuda.h>
 #include <cuda_fp16.h>
 #include <stdio.h>
@@ -8,8 +17,11 @@
 #include <cassert>
 #include <iostream>
 
+#define MAX_OUT_TOKES 1024
 #define MAX_WARP_NUM 32
 #define WARP_SIZE 32
+
+#define MAX_THREADS 1024
 #define SMs 80
 
 #define MAX_REGISTERS 256
@@ -39,10 +51,13 @@ void launch_bias_add(T* input, const T* bias, int hidden_size, int batch_size, c
 
 template <typename T>
 void launch_bias_residual(T* input,
-                          const T* residual,
-                          const T* bias,
-                          int size,
-                          int intermediate_size,
+                          T* output,
+                          T* attn,
+                          T* bias,
+                          T* attn_bias,
+                          int batch,
+                          int hidden_dim,
+                          int mp_size,
                           cudaStream_t stream);
 
 template <typename T>
@@ -67,6 +82,7 @@ void launch_residual_layer_norm(T* norm,
                                 int batch_size,
                                 int hidden_dim,
                                 bool preLN,
+                                bool mlp_after_attn,
                                 cudaStream_t stream);
 template <typename T>
 void launch_dequantize(T* output,
@@ -83,9 +99,12 @@ void launch_gptj_residual_add(T* input,
                               T* output,
                               T* attn,
                               T* bias,
+                              T* attn_bias,
                               int batch,
                               int head_size,
+                              int mp_size,
                               cudaStream_t stream);
+
 template <typename T>
 void launch_apply_rotary_pos_emb(T* mixed_query,
                                  T* key_layer,
@@ -95,4 +114,42 @@ void launch_apply_rotary_pos_emb(T* mixed_query,
                                  unsigned offset,
                                  unsigned num_heads,
                                  unsigned batch,
+                                 bool rotate_half,
+                                 bool rotate_every_two,
                                  cudaStream_t stream);
+
+template <typename T>
+void launch_moe_res_matmul(T* residual,
+                           T* coef,
+                           T* mlp_out,
+                           int seq_len,
+                           int hidden_dim,
+                           cudaStream_t stream);
+
+// 4D transform [0, 1, 2, 3] -> [0, 2, 1, 3]
+template <typename T>
+void launch_transform4d_0213(T* out,
+                             const T* in,
+                             int batch_size,
+                             int heads,
+                             int seq_length,
+                             int hidden_dim,
+                             cudaStream_t stream,
+                             int trans_count);
+template <typename T>
+void launch_bias_add_transform_0213(T* outputs,
+                                    T* vals,
+                                    T* vals1,
+                                    const T* vals2,
+                                    const T* bias,
+                                    int batch_size,
+                                    int seq_length,
+                                    unsigned seq_offset,
+                                    int seq_length1,
+                                    int hidden_dim,
+                                    int heads,
+                                    int rotary_dim,
+                                    bool rotate_half,
+                                    bool rotate_every_two,
+                                    cudaStream_t stream,
+                                    int trans_count);
