@@ -28,6 +28,7 @@ from .utils import log_dist
 from .utils.distributed import init_distributed
 
 from .runtime import zero
+from .runtime import DeepSpeedOptimizer, ZeROOptimizer
 
 from .pipe import PipelineModule
 
@@ -218,24 +219,39 @@ def add_config_arguments(parser):
 
 
 def init_inference(model,
+                   triangular_masking=True,
                    mp_size=1,
+                   training_mp_size=1,
                    mpu=None,
+                   ep_group=None,
+                   expert_mp_group=None,
                    checkpoint=None,
-                   module_key='module',
                    dtype=None,
                    injection_policy=None,
                    replace_method='auto',
                    quantization_setting=None,
                    replace_with_kernel_inject=False,
-                   return_tuple=True):
+                   return_tuple=True,
+                   ep_size=1,
+                   moe=False,
+                   moe_experts=1,
+                   moe_type='standard',
+                   args=None,
+                   enable_cuda_graph=False):
     """Initialize the DeepSpeed InferenceEngine.
 
     Arguments:
         model: Required: nn.module class before apply any wrappers
 
+        triangular_masking: Required: this shows the type of masking for attention scores in transformer layer
+            note that the masking is application specific.
+
         mp_size: Optional: Desired model parallel size, default is 1 meaning no
             model parallelism.
 
+        training_mp_size: Optional: if loading a checkpoint this is the mp size that it was trained with,
+            it may be different than what the mp size that you want to use during inference.
+
         mpu: Optional: A model parallelism unit object that implements
             get_{model,data}_parallel_{rank,group,world_size}()
 
@@ -268,18 +284,25 @@ def init_inference(model,
         __git_branch__),
              ranks=[0])
 
-    if isinstance(model, PipelineModule):
-        raise NotImplementedError("pipeline module support is not implemented yet")
-    else:
-        engine = InferenceEngine(model,
-                                 mp_size,
-                                 mpu,
-                                 checkpoint,
-                                 dtype,
-                                 injection_policy,
-                                 return_tuple,
-                                 replace_method,
-                                 quantization_setting,
-                                 replace_with_kernel_inject)
+    engine = InferenceEngine(model,
+                             triangular_masking,
+                             mp_size,
+                             training_mp_size,
+                             ep_size,
+                             mpu,
+                             ep_group,
+                             expert_mp_group,
+                             checkpoint,
+                             dtype,
+                             injection_policy,
+                             return_tuple,
+                             replace_method,
+                             quantization_setting,
+                             replace_with_kernel_inject,
+                             moe,
+                             moe_experts,
+                             moe_type,
+                             args,
+                             enable_cuda_graph)
 
     return engine
